@@ -1,6 +1,6 @@
 /*
  * hostapd / Kernel driver communication with Devicescape IEEE 802.11 stack
- * Copyright (c) 2002-2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2002-2005, Jouni Malinen <j@w1.fi>
  * Copyright (c) 2003-2004, Instant802 Networks, Inc.
  * Copyright (c) 2005-2006, Devicescape Software, Inc.
  *
@@ -38,8 +38,16 @@
 #include "sta_info.h"
 #include "hw_features.h"
 #include <hostapd_ioctl.h>
-#include <net/d80211_common.h>
-#include <net/d80211_shared.h>
+#include <ieee80211_common.h>
+/* from net/mac80211.h */
+enum {
+	MODE_IEEE80211A = 0 /* IEEE 802.11a */,
+	MODE_IEEE80211B = 1 /* IEEE 802.11b only */,
+	MODE_ATHEROS_TURBO = 2 /* Atheros Turbo mode (2x.11a at 5 GHz) */,
+	MODE_IEEE80211G = 3 /* IEEE 802.11g (and 802.11b compatibility) */,
+	MODE_ATHEROS_TURBOG = 4 /* Atheros Turbo mode (2x.11g at 2.4 GHz) */,
+	NUM_IEEE80211_MODES = 5
+};
 #include "mlme.h"
 
 
@@ -626,23 +634,6 @@ static int i802_send_eapol(void *priv, const u8 *addr, const u8 *data,
 }
 
 
-static int i802_set_assoc_ap(void *priv, const u8 *addr)
-{
-	struct i802_driver_data *drv = priv;
-	struct prism2_hostapd_param param;
-
-	memset(&param, 0, sizeof(param));
-	param.cmd = PRISM2_HOSTAPD_SET_ASSOC_AP_ADDR;
-	memcpy(param.sta_addr, addr, ETH_ALEN);
-	if (hostapd_ioctl(drv, &param, sizeof(param))) {
-		printf("Could not set associated AP address to kernel "
-		       "driver.\n");
-		return -1;
-	}
-	return 0;
-}
-
-
 static int i802_sta_add(const char *ifname, void *priv, const u8 *addr,
 			u16 aid, u16 capability, u8 *supp_rates,
 			size_t supp_rates_len, int flags)
@@ -1156,7 +1147,10 @@ static struct hostapd_hw_modes * i802_get_hw_feature_data(void *priv,
 
 		feature->channels = malloc(clen);
 		feature->rates = malloc(rlen);
-		if (!feature->channels || !feature->rates) {
+		if (!feature->channels || !feature->rates ||
+		    pos + clen + rlen > end) {
+			wpa_printf(MSG_ERROR, "%s: Could not parse data",
+				   __func__);
 			hostapd_free_hw_features(modes, *num_modes);
 			modes = NULL;
 			break;
@@ -2007,7 +2001,6 @@ static const struct driver_ops devicescape_driver_ops = {
 	.sta_remove = i802_sta_remove,
 	.set_ssid = i802_set_ssid,
 	.send_mgmt_frame = i802_send_mgmt_frame,
-	.set_assoc_ap = i802_set_assoc_ap,
 	.sta_add = i802_sta_add,
 	.get_inact_sec = i802_get_inact_sec,
 	.sta_clear_stats = i802_sta_clear_stats,
