@@ -98,7 +98,6 @@ set80211priv(struct madwifi_driver_data *drv, int op, void *data, int len)
 	if (ioctl(drv->ioctl_sock, op, &iwr) < 0) {
 #ifdef MADWIFI_NG
 		int first = IEEE80211_IOCTL_SETPARAM;
-		int last = IEEE80211_IOCTL_KICKMAC;
 		static const char *opnames[] = {
 			"ioctl[IEEE80211_IOCTL_SETPARAM]",
 			"ioctl[IEEE80211_IOCTL_GETPARAM]",
@@ -109,10 +108,10 @@ set80211priv(struct madwifi_driver_data *drv, int op, void *data, int len)
 			"ioctl[IEEE80211_IOCTL_SETCHANLIST]",
 			"ioctl[IEEE80211_IOCTL_GETCHANLIST]",
 			"ioctl[IEEE80211_IOCTL_CHANSWITCH]",
-			NULL,
-			NULL,
+			"ioctl[IEEE80211_IOCTL_GET_APPIEBUF]",
+			"ioctl[IEEE80211_IOCTL_SET_APPIEBUF]",
 			"ioctl[IEEE80211_IOCTL_GETSCANRESULTS]",
-			NULL,
+			"ioctl[IEEE80211_IOCTL_FILTERFRAME]",
 			"ioctl[IEEE80211_IOCTL_GETCHANINFO]",
 			"ioctl[IEEE80211_IOCTL_SETOPTIE]",
 			"ioctl[IEEE80211_IOCTL_GETOPTIE]",
@@ -130,11 +129,10 @@ set80211priv(struct madwifi_driver_data *drv, int op, void *data, int len)
 			NULL,
 			"ioctl[IEEE80211_IOCTL_WDSDELMAC]",
 			NULL,
-			"ioctl[IEEE80211_IOCTL_KICMAC]",
+			"ioctl[IEEE80211_IOCTL_KICKMAC]",
 		};
 #else /* MADWIFI_NG */
 		int first = IEEE80211_IOCTL_SETPARAM;
-		int last = IEEE80211_IOCTL_CHANLIST;
 		static const char *opnames[] = {
 			"ioctl[IEEE80211_IOCTL_SETPARAM]",
 			"ioctl[IEEE80211_IOCTL_GETPARAM]",
@@ -158,7 +156,7 @@ set80211priv(struct madwifi_driver_data *drv, int op, void *data, int len)
 		};
 #endif /* MADWIFI_NG */
 		int idx = op - first;
-		if (first <= op && op <= last &&
+		if (first <= op &&
 		    idx < (int) (sizeof(opnames) / sizeof(opnames[0])) &&
 		    opnames[idx])
 			perror(opnames[idx]);
