@@ -2,6 +2,7 @@
  * hostapd / Driver interaction with Prism54 PIMFOR interface
  * Copyright (c) 2004, Bell Kin <bell_kin@pek.com.tw>
  * based on hostap driver.c, ieee802_11.c
+ * Copyright (c) 2002-2007, Jouni Malinen <j@w1.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -171,7 +172,7 @@ static int prism54_set_sta_authorized(void *priv, const u8 *addr,
 	struct prism54_driver_data *drv = priv;
 	pimdev_hdr *hdr;
 	char *pos;
-	int res;
+
 	hdr = malloc(sizeof(*hdr) + ETH_ALEN);
 	if (hdr == NULL)
 		return -1;
@@ -183,7 +184,7 @@ static int prism54_set_sta_authorized(void *priv, const u8 *addr,
 	}
 	pos = (char *) (hdr + 1);
 	memcpy(pos, addr, ETH_ALEN);
-	res = send(drv->pim_sock, hdr, sizeof(*hdr) + ETH_ALEN, 0);
+	send(drv->pim_sock, hdr, sizeof(*hdr) + ETH_ALEN, 0);
 	prism54_waitpim(priv, hdr->oid, hdr, sizeof(*hdr) + ETH_ALEN, 10);
 	free(hdr);
 	return 0;
@@ -315,7 +316,6 @@ static int prism54_init_1x(void *priv)
 	struct prism54_driver_data *drv = priv;
 	pimdev_hdr *hdr;
 	unsigned long *ul;
-	int ret;
 	int blen = sizeof(*hdr) + sizeof(*ul);
 
 	hdr = malloc(blen);
@@ -326,22 +326,22 @@ static int prism54_init_1x(void *priv)
 	hdr->op = htonl(PIMOP_SET);
 	hdr->oid = htonl(DOT11_OID_EXUNENCRYPTED);
 	*ul = htonl(DOT11_BOOL_TRUE); /* not accept */
-	ret = send(drv->pim_sock, hdr, blen, 0);
+	send(drv->pim_sock, hdr, blen, 0);
 	prism54_waitpim(priv, DOT11_OID_EXUNENCRYPTED, hdr, blen, 10);
 	hdr->op = htonl(PIMOP_SET);
 	hdr->oid = htonl(DOT11_OID_MLMEAUTOLEVEL);
 	*ul = htonl(DOT11_MLME_EXTENDED);
-	ret = send(drv->pim_sock, hdr, blen, 0);
+	send(drv->pim_sock, hdr, blen, 0);
 	prism54_waitpim(priv, DOT11_OID_MLMEAUTOLEVEL, hdr, blen, 10);
 	hdr->op = htonl(PIMOP_SET);
 	hdr->oid = htonl(DOT11_OID_DOT1XENABLE);
 	*ul = htonl(DOT11_BOOL_TRUE);
-	ret = send(drv->pim_sock, hdr, blen, 0);
+	send(drv->pim_sock, hdr, blen, 0);
 	prism54_waitpim(priv, DOT11_OID_DOT1XENABLE, hdr, blen, 10);
 	hdr->op = htonl(PIMOP_SET);
 	hdr->oid = htonl(DOT11_OID_AUTHENABLE);
 	*ul = htonl(DOT11_AUTH_OS); /* OS */
-	ret = send(drv->pim_sock, hdr, blen, 0);
+	send(drv->pim_sock, hdr, blen, 0);
 	prism54_waitpim(priv, DOT11_OID_AUTHENABLE, hdr, blen, 10);
 	free(hdr);
 	return 0;
@@ -407,7 +407,8 @@ static int prism54_flush(void *priv)
 	struct prism54_driver_data *drv = priv;
 	struct obj_mlmeex *mlme;
 	pimdev_hdr *hdr;
-	int ret, i;
+	int ret;
+	unsigned int i;
 	long *nsta;
 	int blen = sizeof(*hdr) + sizeof(*mlme);
 	char *mac_id;
@@ -547,7 +548,6 @@ static int prism54_set_generic_elem(const char *ifname, void *priv,
 				    const u8 *elem, size_t elem_len)
 {
 	struct prism54_driver_data *drv = priv;
-	int res;
 	pimdev_hdr *hdr;
 	char *pos;
 	struct obj_attachment_hdr *attach;
@@ -566,9 +566,9 @@ static int prism54_set_generic_elem(const char *ifname, void *priv,
 	pos = ((char*) attach) + sizeof(*attach);
 	if (elem)
 		memcpy(pos, elem, elem_len);
-	res = send(drv->pim_sock, hdr, blen, 0);
+	send(drv->pim_sock, hdr, blen, 0);
 	attach->type = DOT11_PKT_PROBE_RESP;
-	res = send(drv->pim_sock, hdr, blen, 0);
+	send(drv->pim_sock, hdr, blen, 0);
 	free(hdr);
 	return 0;
 }
@@ -859,15 +859,20 @@ fail:
 static void handle_pim(int sock, void *eloop_ctx, void *sock_ctx)
 {
 	struct prism54_driver_data *drv = eloop_ctx;
-	size_t len;
+	int len;
 	pimdev_hdr *hdr;
 
 	hdr = malloc(PIM_BUF_SIZE);
 	if (hdr == NULL)
 		return;
 	len = recv(sock, hdr, PIM_BUF_SIZE, 0);
+	if (len < 0) {
+		perror("recv");
+		free(hdr);
+		return;
+	}
 	if (len < 8) {
-		printf("handle_pim: too short (%lu)\n", (unsigned long) len);
+		printf("handle_pim: too short (%d)\n", len);
 		free(hdr);
 		return;
 	}
