@@ -1,6 +1,6 @@
 /*
  * hostapd / IEEE 802.11F-2003 Inter-Access Point Protocol (IAPP)
- * Copyright (c) 2002-2006, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2002-2007, Jouni Malinen <j@w1.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -527,10 +527,7 @@ int iapp_reconfig(struct hostapd_data *hapd, struct hostapd_config *oldconf,
 		  struct hostapd_bss_config *oldbss)
 {
 	if (hapd->conf->ieee802_11f != oldbss->ieee802_11f ||
-	    (hapd->conf->iapp_iface && !oldbss->iapp_iface) ||
-	    (!hapd->conf->iapp_iface && oldbss->iapp_iface) ||
-	    (hapd->conf->iapp_iface &&
-	     strcmp(hapd->conf->iapp_iface, oldbss->iapp_iface) != 0)) {
+	    strcmp(hapd->conf->iapp_iface, oldbss->iapp_iface) != 0) {
 
 		iapp_deinit(hapd->iapp);
 		hapd->iapp = NULL;
